<?php declare(strict_types=1);

namespace App\Dto\Service\Registry\V2\AuthorizationService;

final readonly class Scope
{
    public function __construct(
        private string $username,
        private string $repositoryName,
        private array  $actions = [],
    ) { }

    public static function parse(string $scope): Scope
    {
        [$none, $repository, $actions] = explode(':', $scope, 3);
        unset($none);
        [$username, $repositoryName] = explode('/', $repository, 2);

        return new Scope(
            username: $username,
            repositoryName: $repositoryName,
            actions: explode(',', $actions),
        );
    }


    public function getUsername(): string
    {
        return $this->username;
    }

    public function getRepositoryName(): string
    {
        return $this->repositoryName;
    }

    public function getActions(): array
    {
        return $this->actions;
    }
}
