<?php declare(strict_types=1);

namespace App\Http\Controllers\Site;

use App\Dto\QuerySettingsDto;
use App\Http\Requests\Site\TagRepositories\TagsRequest;
use App\Services\Site\TagRepositoryService;
use Illuminate\Http\RedirectResponse;
use Illuminate\View\View;

final class TagRepositoriesController extends Controller
{
    public function __construct(
        private readonly TagRepositoryService $tagRepositoryService,
    ) { }

    public function tags(string $username, string $repositoryName, TagsRequest $request): View
    {
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page:  $request->getDto()->getPage(),
            queryWith: ['repository', 'architecture']
        );
        $result = $this->tagRepositoryService->tags($username, $repositoryName, $querySettingsDto, $request->user());
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('site.repositories.tags', $result->getData());
    }

    public function destroy(string $username, string $repositoryName, int $tagId, TagsRequest $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->tagRepositoryService->destroy($username, $repositoryName, $tagId, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('repository.tags', ['username' => $username, 'repository_name' => $repositoryName])->withSuccess($result->getMessage());
    }
}
