<?php declare(strict_types=1);

namespace App\Http\Middleware;

use Illuminate\Auth\AuthenticationException;
use Illuminate\Http\Request;
use Closure;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Str;

final class RegistryAuth
{
    /**
     * @throws AuthenticationException
     */
    public function handle(Request $request, Closure $next)
    {
        if (RateLimiter::tooManyAttempts(key: 'registry-auth:' . $request->getClientIp(), maxAttempts: 15)) {
            $message = __('http-statuses.' . Response::HTTP_TOO_MANY_REQUESTS, [], 'en');
            \abort(
                new Response($message, Response::HTTP_TOO_MANY_REQUESTS)
            );
        }

        if ($request->hasHeader('authorization') && $this->authenticate($request) === false) {
            RateLimiter::increment(key: 'registry-auth:' . $request->getClientIp(), decaySeconds: 3600);
            throw new \Illuminate\Auth\AuthenticationException();
        }

        return $next($request);
    }

    private function authenticate(Request $request): bool
    {
        $basicHeader = $this->getHeaderBasic($request);
        $authenticated = base64_decode($basicHeader);
        if ($authenticated === false) {
            return false;
        }

        $authenticated = explode(':', $authenticated, 2);
        if (count($authenticated) !== 2) {
            return false;
        }
        [$username, $token] = $authenticated;

        $request->headers->set('authorization', 'Bearer ' . $token);
        Config::set('auth.defaults.guard', 'sanctum');
        if ($request->user()?->username === Str::lower($username)) {
            return true;
        }

        return false;
    }

    private function getHeaderBasic(Request $request): string
    {
        $basicHeader = $request->header('authorization');
        $position = strrpos($basicHeader, 'Basic ');

        if ($position !== false) {
            return substr($basicHeader, $position + 6);
        }

        return $basicHeader;
    }
}
