<?php declare(strict_types=1);

namespace App\Services\Registry\V2\Api;

use App\Contracts\ServiceResultError;
use App\ServiceResults\ServiceResultArray;
use App\Services\Service;
use Illuminate\Support\Facades\Log;

final class ApiService extends Service
{
    public function __construct(
        private readonly RequestCommandHandler   $requestCommandHandler,
        private readonly GenerateJwtTokenCommand $generateJwtTokenCommand,
    ) { }

    public function getManifest(string $repositoryRegistry, string $reference): ServiceResultError | ServiceResultArray
    {
        $access = [
            $this->getAccessRepository($repositoryRegistry, ['pull']),
        ];
        $username = '';

        try {
            $token = $this->generateJwtTokenCommand->execute($access, $username);

            $path = $repositoryRegistry . '/manifests/' . $reference;
            $result = $this->requestCommandHandler->handleGet($path, $token);
            Log::info($result);
        } catch (\Throwable $e) {
            report($e);
            return $this->errService($e->getMessage());
        }

        return $this->result($result);
    }

    public function getBlob(string $repositoryRegistry, string $digest): ServiceResultError | ServiceResultArray
    {
        $access = [
            $this->getAccessRepository($repositoryRegistry, ['pull']),
        ];
        $username = '';

        try {
            $token = $this->generateJwtTokenCommand->execute($access, $username);

            $path = $repositoryRegistry . '/blobs/' . $digest;
            $result = $this->requestCommandHandler->handleGet($path, $token);
            Log::info($result);
        } catch (\Throwable $e) {
            report($e);
            return $this->errService($e->getMessage());
        }

        return $this->result($result);
    }

    private function getAccessRepository(string $repository, array $actions): array
    {
        return [
            'type'    => 'repository',
            'name'    => $repository,
            'actions' => $actions,
        ];
    }
}
